@@ -38,6 +38,7 @@
 #include <errno.h>
 
 #include "glib.h"
+#include "gstdio.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -318,21 +319,165 @@ find_first_that(gpointer key,
 }
 
 
+static gboolean
+test_g_mkdir_with_parents_1 (const gchar *base)
+{
+  char *p0 = g_build_filename (base, "fum", NULL);
+  char *p1 = g_build_filename (p0, "tem", NULL);
+  char *p2 = g_build_filename (p1, "zap", NULL);
+  FILE *f;
+
+  g_remove (p2);
+  g_remove (p1);
+  g_remove (p0);
+
+  if (g_file_test (p0, G_FILE_TEST_EXISTS))
+    {
+      g_print ("failed, %s exists, cannot test g_mkdir_with_parents\n", p0);
+      return FALSE;
+    }
+
+  if (g_file_test (p1, G_FILE_TEST_EXISTS))
+    {
+      g_print ("failed, %s exists, cannot test g_mkdir_with_parents\n", p1);
+      return FALSE;
+    }
+
+  if (g_file_test (p2, G_FILE_TEST_EXISTS))
+    {
+      g_print ("failed, %s exists, cannot test g_mkdir_with_parents\n", p2);
+      return FALSE;
+    }
+
+  if (g_mkdir_with_parents (p2, 0666) == -1)
+    {
+      g_print ("failed, g_mkdir_with_parents(%s) failed: %s\n", p2, g_strerror (errno));
+      return FALSE;
+    }
+
+  if (!g_file_test (p2, G_FILE_TEST_IS_DIR))
+    {
+      g_print ("failed, g_mkdir_with_parents(%s) succeeded, but %s is not a directory\n", p2, p2);
+      return FALSE;
+    }
+
+  if (!g_file_test (p1, G_FILE_TEST_IS_DIR))
+    {
+      g_print ("failed, g_mkdir_with_parents(%s) succeeded, but %s is not a directory\n", p2, p1);
+      return FALSE;
+    }
+
+  if (!g_file_test (p0, G_FILE_TEST_IS_DIR))
+    {
+      g_print ("failed, g_mkdir_with_parents(%s) succeeded, but %s is not a directory\n", p2, p0);
+      return FALSE;
+    }
+
+  g_rmdir (p2);
+  if (g_file_test (p2, G_FILE_TEST_EXISTS))
+    {
+      g_print ("failed, did g_rmdir(%s), but %s is still there\n", p2, p2);
+      return FALSE;
+    }
+
+  g_rmdir (p1);
+  if (g_file_test (p1, G_FILE_TEST_EXISTS))
+    {
+      g_print ("failed, did g_rmdir(%s), but %s is still there\n", p1, p1);
+      return FALSE;
+    }
+
+  f = g_fopen (p1, "w");
+  if (f == NULL)
+    {
+      g_print ("failed, couldn't create file %s\n", p1);
+      return FALSE;
+    }
+  fclose (f);
+  
+  if (g_mkdir_with_parents (p1, 0666) == 0)
+    {
+      g_print ("failed, g_mkdir_with_parents(%s) succeeded, even if %s is a file\n", p1, p1);
+      return FALSE;
+    }
+
+  if (g_mkdir_with_parents (p2, 0666) == 0)
+    {
+      g_print ("failed, g_mkdir_with_parents(%s) succeeded, even if %s is a file\n", p2, p1);
+      return FALSE;
+    }
+
+  g_remove (p2);
+  g_remove (p1);
+  g_remove (p0);
+
+  return TRUE;
+}
+
+static gboolean
+test_g_mkdir_with_parents (void)
+{
+  g_print ("checking g_mkdir_with_parents()...");
+  if (!test_g_mkdir_with_parents_1 ("hum"))
+    return FALSE;
+  g_remove ("hum");
+  if (!test_g_mkdir_with_parents_1 ("hii///haa/hee"))
+    return FALSE;
+  g_remove ("hii/haa/hee");
+  g_remove ("hii/haa");
+  g_remove ("hii");
+  if (!test_g_mkdir_with_parents_1 (g_get_current_dir ()))
+    return FALSE;
+
+  return TRUE;
+}
+
+static void
+test_g_parse_debug_string (void)
+{
+  GDebugKey keys[3] = { 
+    { "foo", 1 },
+    { "bar", 2 },
+    { "baz", 4 }
+  };
+  guint n_keys = 3;
+  guint result;
+  
+  result = g_parse_debug_string ("bar:foo:blubb", keys, n_keys);
+  g_assert (result == 3);
+
+  result = g_parse_debug_string (":baz::_E@~!_::", keys, n_keys);
+  g_assert (result == 4);
+
+  result = g_parse_debug_string ("", keys, n_keys);
+  g_assert (result == 0);
+
+  result = g_parse_debug_string (" : ", keys, n_keys);
+  g_assert (result == 0);
+}
+
+
 int
 main (int   argc,
       char *argv[])
 {
+  const gchar *s;
+  gchar **sv;
   GList *list, *t;
   GSList *slist, *st;
   GHashTable *hash_table;
   GMemChunk *mem_chunk;
   GStringChunk *string_chunk;
   GTimer *timer, *timer2;
+  gdouble elapsed;
+  gulong elapsed_usecs;
   gint nums[10] = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9 };
   gint morenums[10] = { 8, 9, 7, 0, 3, 2, 5, 1, 4, 6};
   gchar *string;
   gint value = 120; 
-  gint *pvalue=NULL; 
+  gint *pvalue=NULL;
+  GTimeVal ref_date, date;
+  gchar *date_str;
   
   gchar *mem[10000], *tmp_string = NULL, *tmp_string_2;
   gint i, j;
@@ -434,8 +579,22 @@ main (int   argc,
   g_free (string);
   g_print ("user: %s\n", g_get_user_name ());
   g_print ("real: %s\n", g_get_real_name ());
-  g_print ("home: %s\n", g_get_home_dir ());
+  g_print ("host: %s\n", g_get_host_name ());
+  s = g_get_home_dir ();
+  g_print ("home: %s\n", s ? s : "NULL!");
+  s = g_get_user_data_dir ();
+  g_print ("user_data: %s\n", s ? s : "NULL!");
+  s = g_get_user_config_dir ();
+  g_print ("user_config: %s\n", s ? s : "NULL!");
+  s = g_get_user_cache_dir ();
+  g_print ("user_cache: %s\n", s ? s : "NULL!");
+  sv = (gchar **) g_get_system_data_dirs ();
+  g_print ("system_data: %s\n", sv ? g_strjoinv (G_SEARCHPATH_SEPARATOR_S, sv) : "NULL!");
+  sv = (gchar **) g_get_system_config_dirs ();
+  g_print ("system_config: %s\n", s ? g_strjoinv (G_SEARCHPATH_SEPARATOR_S, sv) : "NULL!");
   g_print ("tmp-dir: %s\n", g_get_tmp_dir ());
+  sv = (gchar **) g_get_language_names ();
+  g_print ("languages: %s\n", s ? g_strjoinv (":", sv) : "NULL!");
 
   /* type sizes */
   g_print ("checking size of gint8: %"    G_GSIZE_FORMAT, sizeof (gint8));
@@ -464,7 +623,6 @@ main (int   argc,
   string = g_path_get_basename ("/foo/file");
   g_assert (strcmp (string, "file") == 0);
   g_free (string);
-  g_print ("ok\n");
 #endif
 
   g_print ("checking g_path_get_dirname()...");
@@ -508,6 +666,9 @@ main (int   argc,
   if (n_skip_root_checks)
     g_print ("ok\n");
 
+  if (test_g_mkdir_with_parents ())
+    g_print ("ok\n");
+
   g_print ("checking doubly linked lists...");
 
   list = NULL;
@@ -996,16 +1157,20 @@ main (int   argc,
   g_usleep((29*G_USEC_PER_SEC)/5); /* run timer for 5.8 seconds */
   g_timer_stop(timer);
 
-  g_print ("\t=> total elapsed = %.2f seconds (should be: 9.00 seconds)\n\n", g_timer_elapsed(timer, NULL));
+  elapsed = g_timer_elapsed (timer, &elapsed_usecs);
+  g_print ("\t=> timer = %.6f = %d.%06ld (should be: 9.000000) (%.6f off)\n", elapsed, (int) elapsed, elapsed_usecs, ABS (elapsed - 9.));
 
-  if (g_timer_elapsed(timer, NULL) > 8.8 && g_timer_elapsed(timer, NULL) < 9.2)
+  if (elapsed > 8.8 && elapsed < 9.2)
     g_print ("g_timer_continue ... ok\n\n");
   else
     g_print ("g_timer_continue ... ***** FAILED *****\n\n");
 
   g_timer_stop(timer2);
 
-  if (g_timer_elapsed(timer2, NULL) > (8.8+6.5) && g_timer_elapsed(timer2, NULL) < (9.2+6.5))
+  elapsed = g_timer_elapsed(timer2, &elapsed_usecs);
+  g_print ("\t=> timer2 = %.6f = %d.%06ld (should be: %.6f) (%.6f off)\n\n", elapsed, (int) elapsed, elapsed_usecs, 9.+6.5, ABS (elapsed - (9.+6.5)));
+
+  if (elapsed > (8.8+6.5) && elapsed < (9.2+6.5))
     g_print ("timer2 ... ok\n\n");
   else
     g_print ("timer2 ... ***** FAILED *****\n\n");
@@ -1013,6 +1178,37 @@ main (int   argc,
   g_timer_destroy(timer);
   g_timer_destroy(timer2);
 
+#define REF_SEC_UTC  320063760
+#define REF_STR_UTC  "1980-02-22T10:36:00Z"
+#define REF_STR_CEST "1980-02-22T12:36:00+02:00"
+#define REF_STR_EST  "1980-02-22T05:36:00-05:00"
+
+  g_print ("checking g_time_val_from_iso8601...\n");
+  ref_date.tv_sec = REF_SEC_UTC;
+  ref_date.tv_usec = 0;
+  g_assert (g_time_val_from_iso8601 (REF_STR_UTC, &date) != FALSE);
+  g_print ("\t=> UTC stamp = %ld (should be: %ld) (%ld off)\n", date.tv_sec, ref_date.tv_sec, date.tv_sec - ref_date.tv_sec);
+  g_assert (date.tv_sec == ref_date.tv_sec);
+
+  g_assert (g_time_val_from_iso8601 (REF_STR_CEST, &date) != FALSE);
+  g_print ("\t=> CEST stamp = %ld (should be: %ld) (%ld off)\n", date.tv_sec, ref_date.tv_sec, date.tv_sec - ref_date.tv_sec);
+  g_assert (date.tv_sec == ref_date.tv_sec);
+  
+  g_assert (g_time_val_from_iso8601 (REF_STR_EST, &date) != FALSE);
+  g_print ("\t=> EST stamp = %ld (should be: %ld) (%ld off)\n", date.tv_sec, ref_date.tv_sec, date.tv_sec - ref_date.tv_sec);
+  g_assert (date.tv_sec == ref_date.tv_sec);
+  g_print ("ok\n");
+  
+  g_print ("checking g_time_val_to_iso8601...\n");
+  ref_date.tv_sec = REF_SEC_UTC;
+  ref_date.tv_usec = 1;
+  date_str = g_time_val_to_iso8601 (&ref_date);
+  g_assert (date_str != NULL);
+  g_print ("\t=> date string = %s (should be: %s)\n", date_str, REF_STR_UTC);
+  g_assert (strcmp (date_str, REF_STR_UTC) == 0);
+  g_free (date_str);
+  g_print ("ok\n");
+
   g_print ("checking g_ascii_strcasecmp...");
   g_assert (g_ascii_strcasecmp ("FroboZZ", "frobozz") == 0);
   g_assert (g_ascii_strcasecmp ("frobozz", "frobozz") == 0);
@@ -1269,6 +1465,8 @@ main (int   argc,
   g_print ("found more.com as %s\n", g_find_program_in_path ("more.com"));
   g_print ("found regedit as %s\n", g_find_program_in_path ("regedit"));
 
+  g_print ("a Win32 error message: %s\n", g_win32_error_message (2));
+
 #endif
 
   g_print ("checking file functions...\n");
@@ -1348,7 +1546,9 @@ main (int   argc,
   close (fd);
   g_clear_error (&error);
   remove (name_used);
-  
+
+  test_g_parse_debug_string ();
+
   return 0;
 }
 
